const express = require('express');
const path = require('path');
// const favicon = require('serve-favicon');
const logger = require('morgan');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const cors = require('cors');

require('dotenv').config(); //dotenv-moduuli tarvitaan jos aiotaan käyttää .env -filua
require('./dbconnection'); //kantayhteys

const users = require('./routes/users');
// const api = require('./routes/api');

const app = express();

// cors avaa yhteyden palvelinsovelluksen ja asiakassovelluksen välille
// jos nämä sijaitsevat eri palvelimilla
const corsOptions = {
  origin: 'http://localhost:4200', // frontin osoite mihin saadaan olla yhteydessä
  optionsSuccessStatus: 200,
};

app.use(cors(corsOptions));

// uncomment after placing your favicon in /public
// app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(
  bodyParser.urlencoded({
    extended: false,
  })
);
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use(cors()); //cors kaikille reiteille

app.use('/users', users);
// app.use('/api', api);

// catch 404 and forward to error handler
app.use(function (req, res, next) {
  const err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handler
app.use(function (err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

module.exports = app;
